@echo off
rem  TinyCC 1.5 main script for windows
rem  by Chris Biemann, Jun 2006
rem  biem@informatik.uni-leipzig.de
rem 
rem  for indexing and computing co-occurrences for small and medium-sized corpora

rem  this script takes 2 parameter files:
rem  - a one-sentence-per-line text file %1 that is found in subdirectory "data"
rem  - a multi word unit list, one MWU per line %2 (or "none")

rem  and writes 5 files:
rem  - %1.sentences:   s_id <tab> sentence
rem  - %1.wordlist     w_id <tab> word <tab> frequency
rem  - %1.index        w_id <tab> s_id <tab> pos [- if part of following MWU]
rem  - %1.scooc        w1_id <tab> w2_id <tab> freq <tab> sig
rem  - %1.nbcooc        w_1id <tab> w2_id <tab> freq <tab> sig

rem  Memory max usage in MB (approximate)
set MAXMEM=500
rem  min frequency for scoocs
set SMINFREQ=2
rem  min sig for scooc
set SMINSIG=6.63
rem  min freq for nbcooc
set NMINFREQ=2
rem  min sig for NBcooc
set NMINSIG=3.84
rem  number of digits after .
set DIGITS=2

echo settings: Textfile = %1, Multiword list: %2 , Memory= %MAXMEM%, Sminfreq= %SMINFREQ%, Sminsig = %SMINSIG%, Nminfreq = %NMINFREQ%, Nminsig= %NMINFREQ%, fraction rounding after %DIGITS% digits.

     
echo "Collecting texts and converting...
java -jar text2satz.jar -a abbrev -n -e data\%1 
rem -n: Newline is always sentence break. -a: abbreviation list. These words can be followed by "." without a sentence split. 
move db.s %1.s
del sources.txt
del sentsrc.txt
echo "Numbering sentences..."
perl numberIt.pl %1.s > %1.sentences
echo "Tokenizing ... "
perl tokenize.pl %1.sentences %1.tok
perl tok_multiwords.pl %2 %2.tok
echo "Collecting single words ..."
perl freqSingle.pl %1.tok %1.singlewords
echo "Collecting and counting all words .."
perl freqMulti.pl %1.singlewords %2.tok %1.tok %1.words
echo "Indexing ..."
perl index_wl.pl %1.words %1.tok %1
echo "Calculating neighbour co-occurrences"
perl nbcooc.pl %1.wordlist_tok %1.index %MAXMEM% %NMINFREQ% %NMINSIG% %DIGITS% %1.nbcooc
echo "Calculating sentence co-occurrences"
echo "- collecting pairwise frequencies"
perl sfreq.pl %1.wordlist_tok %1.index %MAXMEM% %SMINFREQ% %1
echo "- sorting pairwise frequencies"
sort %1.sfreqtemp > %1.sfreqtempsort
echo "- adding frequencies"
perl add3col_sym.pl %1.sfreqtempsort %SMINFREQ% %1.sfreq
echo "- computing significances"
perl ssig.pl %1.wordlist_tok %1.index %1.sfreq %SMINSIG% %DIGITS% %1.scooc
echo "De-tokenizing word list"
perl detok_multiwords.pl %1.wordlist_tok %1.wordlist
echo "Cleaning up ..."


del %1.tok %1.singlewords %1.words %2.tok %1.sfreq %1.sfreqtemp %1.index.nbfreq %1.index.nbfreqtemp %1.wordlist_tok %1.wordlist_tok.unique %1.index.scount %1.sfreqtempsort
echo "Done !"

